--
-- MP3 Player, VHDL version (http://www.wiresncode.com/projects/pjrcmp3/fpga)
-- Copyright (c) 2000-1, PJRC.COM, LLC (paul_at_pjrc.com)
-- Copyright (c) 2002, Tom Parker (tparker_at_netspace.net.au)
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity addr_dec is 
  port (adr: in std_logic_vector(15 downto 0);
	reset, adr_en, clk: in std_logic;
	a_intr_mask, a_intr_src, a_memcpy_ack, a_sta013_ack, a_idexfer_ack,
	a_memcpy_go, a_sta013_go, a_idexfer_go, a_dram, pg_mem, sh_mem,
	a_reset_ide, a_buf_ide, a_ide, reg_access: out std_logic);
end addr_dec;

architecture behavior of addr_dec is
  signal address : bit_vector(15 downto 0);
begin
  address <= to_bitvector(adr);
  p1: process(clk, reset, adr_en, address)
  begin
    if reset = '1' then
      a_intr_mask <= '0';
      a_intr_src <= '0';
      a_memcpy_ack <= '0';
      a_sta013_ack <= '0';
      a_idexfer_ack <= '0';
      a_memcpy_go <= '0';
      a_sta013_go <= '0';
      a_idexfer_go <= '0';
      a_dram <= '0';
      pg_mem <= '0';
      sh_mem <= '0';
      a_reset_ide <= '0';
      a_buf_ide <= '0';
      a_ide <= '0';
      reg_access <= '0';
    else if clk'event and clk = '1' then
           if adr_en = '1' then
             -- dram mapped 0000 to efff
             if address >= x"0000" and address <= x"efff" then
               a_dram <= '1';
             else
               a_dram <= '0';
             end if;
             -- pg_mem mapped ff00 to ff1f
             if address >= x"ff00" and address <= x"ff1f" then
               pg_mem <= '1';
             else
               pg_mem <= '0';
             end if;
             -- sh_mem mapped ff20 to ff3f
             if address >= x"ff20" and address <= x"ff3f" then
               sh_mem <= '1';
             else
               sh_mem <= '0';
             end if;
             -- a_reset_ide mapped ff40
             if address = x"ff40" then
               a_reset_ide <= '1';
             else
               a_reset_ide <= '0';
             end if;
             -- a_buf_ide mapped ff42 to ff43
             if address >= x"ff42" and address <= x"ff43" then
               a_buf_ide <= '1';
             else
               a_buf_ide <= '0';
             end if;
             -- a_intr_src mapped ff50
             if address = x"ff50" then
               a_intr_src <= '1';
             else
               a_intr_src <= '0';
             end if;
             -- a_intr_mask mapped ff51
             if address = x"ff51" then
               a_intr_mask <= '1';
             else
               a_intr_mask <= '0';
             end if;
             -- a_idexfer_go mapped ff58
             if address = x"ff58" then
               a_idexfer_go <= '1';
             else
               a_idexfer_go <= '0';
             end if;
             -- a_sta013_go mapped ff59
             if address = x"ff59" then
               a_sta013_go <= '1';
             else
               a_sta013_go <= '0';
             end if;
             -- a_memcpy_go mapped ff5a
             if address = x"ff5a" then
               a_memcpy_go <= '1';
             else
               a_memcpy_go <= '0' ;
             end if;
             -- a_idexfer_ack mapped ff5c
             if address = x"ff5c" then
               a_idexfer_ack <= '1';
             else
               a_idexfer_ack <= '0';
             end if;
             -- a_sta013_ack mapped ff5d
             if address = x"ff5d" then
               a_sta013_ack <= '1';
             else
               a_sta013_ack <= '0';
             end if;
             -- a_memcpy_ack mapped ff5e
             if address = x"ff5e" then
               a_memcpy_ack <= '1';
             else
               a_memcpy_ack <= '0';
             end if;
             -- ide mapped ff60 to ff7f
             if address >= x"ff60" and address <= x"ff7f" then
               a_ide <= '1';
             else
               a_ide <= '0';
             end if;
             -- reg_access all but ide direct access
             if (address >= x"ff00" and address <= x"ff5f") or (address >= x"ffc0" and address <= x"ffdf") then
               reg_access <= '1';
             else
               reg_access <= '0';
             end if;
           end if;
    end if;
  end if;
end process p1;
end behavior; 
