--
-- MP3 Player, VHDL version (http://www.wiresncode.com/projects/pjrcmp3/fpga)
-- Copyright (c) 2000-1, PJRC.COM, LLC (paul_at_pjrc.com)
-- Copyright (c) 2002, Tom Parker (tparker_at_netspace.net.au)
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity addresslatch is
  port(din : in std_logic_vector(7 downto 0);
       ale : in std_logic;
       clk : in std_logic;
       address_raw : in std_logic_vector(15 downto 8);
       reset : in std_logic;
       adr_en : out std_logic;
       adr : out std_logic_vector(15 downto 0));
end addresslatch;

architecture behaviour of addresslatch is
  signal i_en : std_logic;
  signal a_en : std_logic;
begin
  p1: process(clk, address_raw, a_en, reset, ale, din, i_en)
  begin
    -- outputs
    adr_en <= a_en;
    
    -- falling edge
    if reset = '1' then
      i_en <= '0';
      a_en <= '0';
    elsif (clk'event and clk = '0') then
      i_en <= ale;
      a_en <= i_en;
    end if;
    
    -- rising edge
    if reset = '1' then
      adr <= (others => '0');
      adr_en <= '0';
    elsif (clk'event and clk = '1') then
      if i_en = '1' then
        adr(15 downto 8) <= address_raw(15 downto 8);
        adr(7 downto 0) <= din(7 downto 0);
      end if;
    end if;        
  end process P1;
end behaviour;


