--
-- MP3 Player, VHDL version (http://www.wiresncode.com/projects/pjrcmp3/fpga)
-- Copyright (c) 2000-1, PJRC.COM, LLC (paul_at_pjrc.com)
-- Copyright (c) 2002, Tom Parker (tparker_at_netspace.net.au)
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--

-- block controls the ide/dram address/control bus
-- The resulting address bus is connected to both the dram address and the ide
-- address/chip select pins.
-- A3 is connected to ide cs0 (pin 37)
-- A5 is connected to ide cs1 (pin 38)
-- A6 is connected to ide a2 (pin 36)
-- A8 is connected to ide a1 (pin 33)
-- A10 is connected to ide a0 (pin 35)

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity idedramaddress is
  port(clk : in std_logic;
       reset : in std_logic;

       -- INPUTS
       memoryaddress : in std_logic_vector(15 downto 0);
       drampagedata : in std_logic_vector(15 downto 0);
       use_row : in std_logic;
       ide_addr : in std_logic;
       ide_addr_zero : in std_logic;
       all_rows : in std_logic;
       dram_ras : in std_logic;
       idedramaddrenable : in std_logic;
       
       -- OUTPUTS
       dramrowaddress : out std_logic_vector(3 downto 0);
       idedramaddr : out std_logic_vector(10 downto 0));
end idedramaddress;

architecture behavior of idedramaddress is
  signal address : std_logic_vector(10 downto 0);
  signal ras : std_logic_vector(3 downto 0);
begin
  -- RAS logic
  dramrowaddress(0) <= '0' when ras(0) = '1' and dram_ras = '1' else '1';
  dramrowaddress(1) <= '0' when ras(1) = '1' and dram_ras = '1' else '1';
  dramrowaddress(2) <= '0' when ras(2) = '1' and dram_ras = '1' else '1';
  dramrowaddress(3) <= '0' when ras(3) = '1' and dram_ras = '1' else '1';
  
  p1 : process(clk, reset, use_row, all_rows, drampagedata(9), drampagedata(10))
  begin
    if reset = '1' then
      ras <= (others => '0');
    else
      if clk'event and clk = '1' then
        if use_row = '1' then
          ras(0) <= all_rows or (not drampagedata(9) and not drampagedata(10));
          ras(1) <= all_rows or (not drampagedata(9) and drampagedata(10));
          ras(2) <= all_rows or (drampagedata(9) and not drampagedata(10));
          ras(3) <= all_rows or (drampagedata(9) and drampagedata(10));
        end if;
      end if;
    end if;
  end process p1;

  p2 : process(clk, reset, idedramaddrenable, address)
  begin
    if reset = '1' then
      idedramaddr <= (others => '0');
    else
      if clk'event and clk = '1' then
        if idedramaddrenable = '1' then
          idedramaddr <= address;
        end if;
      end if;
    end if;
  end process p2;
  
  -- Address/Control logic 
  address(0) <= '0' when ide_addr = '1' else memoryaddress(11) when use_row = '1' else memoryaddress(1);
  address(1) <= '0' when ide_addr = '1' else drampagedata(0) when use_row = '1' else memoryaddress(2);
  address(2) <= '0' when ide_addr = '1' else drampagedata(1) when use_row = '1' else memoryaddress(3);
  -- address(3) is also ide cs0 (ide pin 37)
  address(3) <= '0' when ide_addr_zero = '1' else memoryaddress(4) when ide_addr = '1' else drampagedata(2) when use_row = '1' else memoryaddress(4);
  address(4) <= '0' when ide_addr = '1' else drampagedata(3) when use_row = '1' else memoryaddress(5);
  -- address(5) is also ide cs1 (ide pin 38)
  address(5) <= '1' when ide_addr_zero = '1' else not memoryaddress(4) when ide_addr = '1' else drampagedata(4) when use_row = '1' else memoryaddress(6);
  -- address(6) is also ide a2 (ide pin 36)
  address(6) <= '0' when ide_addr_zero = '1' else memoryaddress(3) when ide_addr = '1' else drampagedata(5) when use_row = '1' else memoryaddress(7);
  address(7) <= '0' when ide_addr = '1' else drampagedata(6) when use_row = '1' else memoryaddress(8);
  -- address(8) is also ide a1 (ide pin 33)
  address(8) <= '0' when ide_addr_zero = '1' else memoryaddress(2) when ide_addr = '1' else drampagedata(7) when use_row = '1' else memoryaddress(9);
  address(9) <= '0' when ide_addr = '1' else drampagedata(8) when use_row = '1' else memoryaddress(10);
  -- address(10) is also ide a0 (ide pin 35)
  address(10) <= '0' when ide_addr_zero = '1' else memoryaddress(1) when ide_addr = '1' else drampagedata(12) when use_row = '1' else drampagedata(11);
end behavior;
