--
-- MP3 Player, VHDL version (http://www.wiresncode.com/projects/pjrcmp3/fpga)
-- Copyright (c) 2000-1, PJRC.COM, LLC (paul_at_pjrc.com)
-- Copyright (c) 2002, Tom Parker (tparker_at_netspace.net.au)
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--

-- This block monitors cpu external instruction fetches looking for external
-- dram accesses.  This is done by examines the bus for binary instruction
-- data.  When an external access is found the dmaok output is made inactive
-- to allow the cpu bus access to the dram.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity instructionmonitor is 
  port (clk : in std_logic;
        reset : in std_logic;
        psen : in std_logic;
        rddone : in std_logic;
        wrdone : in std_logic;
        cpudatain : in std_logic_vector(7 downto 0);
        dmaok : out std_logic);
end instructionmonitor;

architecture behavior of instructionmonitor is
  signal psen1, psen2, psen3, psen4 : std_logic;
  signal psenfallingedge, psenearlyfallingedge : std_logic;
  signal programdata : integer range 255 downto 0;
  signal dramaccess : std_logic;
begin
  -- dmaok
  countenable: process(clk, dramaccess, psenearlyfallingedge, reset, rddone, wrdone)
  begin
    if reset = '1' or rddone = '1' or wrdone = '1' then
      dmaok <= '1';
    elsif clk'event and clk = '1' then
      if psenearlyfallingedge = '1' then
        dmaok <= not dramaccess;
      end if;
    end if;
  end process countenable;
  
  -- examine program data
  dramaccess <= '1' when (programdata = 16#F0# or
                          programdata = 16#E0# or
                          programdata = 16#F2# or
                          programdata = 16#F3# or
                          programdata = 16#E2# or
                          programdata = 16#E3#) else '0';

  -- sample program data
  program: process(clk, reset, psenfallingedge, cpudatain)
  begin
    if reset = '1' then
      programdata <= 0;
    elsif clk'event and clk = '1' then
      if psenfallingedge = '1' then
        programdata <= conv_integer(cpudatain);
      end if;
    end if;
  end process program;

  -- psen edge detector
  psenfallingedge <= '1' when psen4 = '1' and psen3 = '0' else '0';
  psenearlyfallingedge <= '1' when  psen2 = '1' and psen1 = '0' else '0';

  p1: process(clk, reset, psen)
  begin
    if reset = '1' then
      psen1 <= '0';
    elsif clk'event and clk = '1' then
      psen1 <= psen;
    end if;
  end process p1;

  p2: process(clk, reset, psen1)
  begin
    if reset = '1' then
      psen2 <= '0';
    elsif clk'event and clk = '1' then
      psen2 <= psen1;
    end if;
  end process p2;

  p3: process(clk, reset, psen2)
  begin
    if reset = '1' then
      psen3 <= '0';
    elsif clk'event and clk = '1' then
      psen3 <= psen2;
    end if;
  end process p3;

  p4: process(clk, reset, psen3)
  begin
    if reset = '1' then
      psen4 <= '0';
    elsif clk'event and clk = '1' then
      psen4 <= psen3;
    end if;
  end process p4;
end behavior; 
