--
-- MP3 Player, VHDL version (http://www.wiresncode.com/projects/pjrcmp3/fpga)
-- Copyright (c) 2000-1, PJRC.COM, LLC (paul_at_pjrc.com)
-- Copyright (c) 2002, Tom Parker (tparker_at_netspace.net.au)
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--

--------------------------------------------------------------------------------------
-- Multiplexed 16x16 RAM
--------------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

library unisim;
use unisim.all;

entity muxram is
--  generic (bitmap : std_logic_vector(15 downto 0);
  port(
    -- INPUTS
    din		: in std_logic_vector(15 downto 0);
    address_a, address_b  : in std_logic_vector(3 downto 0);
    we_lsb, we_msb, wclk : in std_logic;
    reg_access, dma_addr : in std_logic;
    a0 : in std_logic;
    dout_en : in std_logic;

    -- OUTPUTS
    dout : out std_logic_vector(7 downto 0);
    q : out std_logic_vector(15 downto 0));
end muxram;

architecture behavior of muxram is

  component RAM16X1S
  port (D    : in std_ulogic;
        WE   : in std_ulogic;
        WCLK : in std_ulogic;
        A0   : in std_ulogic;
        A1   : in std_ulogic;
        A2   : in std_ulogic;
        A3   : in std_ulogic;

        O    : out std_ulogic);
  end component;

  signal ram_dout : std_logic_vector(15 downto 0);
  signal address : std_logic_vector(3 downto 0);

begin
  ram1 : for n in 15 downto 8 generate
    ram1_n: RAM16X1S port map (D => din(n), WE => we_msb, WCLK => wclk,
                                    A0 => address(0), A1 => address(1), A2 => address(2), A3 => address(3), 
                                    O => ram_dout(n));
  end generate ram1;

  ram0 : for n in 7 downto 0 generate
    ram0_n: RAM16X1S port map (D => din(n), WE => we_lsb, WCLK => wclk,
                                    A0 => address(0), A1 => address(1), A2 => address(2), A3 => address(3), 
                                    O => ram_dout(n));
  end generate ram0;

  q <= ram_dout;
  
  -- address multiplexer
  address <= address_a when (reg_access = '1' and dma_addr = '0') else address_b;

  -- 8 bit from 16 bit selector with enable
  dout <= ram_dout(7 downto 0) when (dout_en = '1' and a0 = '0') else
	  ram_dout(15 downto 8) when (dout_en = '1' and a0 = '1') else
	  "ZZZZZZZZ";
  
end behavior;



