--
-- MP3 Player, VHDL version (http://www.wiresncode.com/projects/pjrcmp3/fpga)
-- Copyright (c) 2000-1, PJRC.COM, LLC (paul_at_pjrc.com)
-- Copyright (c) 2002, Tom Parker (tparker_at_netspace.net.au)
--
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License
-- as published by the Free Software Foundation; either version 2
-- of the License, or (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
--

-- This block generates the data output to the sta013.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity sta013driver is 
  port (clk : in std_logic;
        reset : in std_logic;
        d16_in : in std_logic_vector(15 downto 0);
        sta013dataload : in std_logic;
        sta013request : in std_logic;
        sta013data : out std_logic;
        sta013clock : out std_logic;
        sta013idle : out std_logic);
end sta013driver;

architecture behavior of sta013driver is

  type states is (IDLE, LOADING, WAITING, CLOCKING);
  
  signal dataout : std_logic_vector(16 downto 0);
  signal datain : std_logic_vector(16 downto 0);
  signal bitcount : std_logic_vector(4 downto 0);
  signal state : states;
  signal bitcountzero : std_logic;
  
begin

  -- bit counter
  bitcountzero <= '1' when bitcount = "00000" else '0';
  sta013clock <= bitcount(0);
  counter : process(clk, reset, state, bitcount, bitcountzero)
  begin
    if reset = '1' or state = IDLE then
      bitcount <= "11111";
    elsif clk'event and clk = '1' then
      if state = CLOCKING and bitcountzero = '0' then
        bitcount <= bitcount - 1;
      end if;
    end if;	
  end process counter;

  -- state machine
  sta013idle <= '1' when state = IDLE else '0';
  statemachine : process(clk, reset, state, sta013dataload, sta013request, bitcountzero)
  begin
    if reset = '1' then
      state <= IDLE;
    elsif clk'event and clk = '1' then
      if sta013dataload = '1' and state = IDLE then
        state <= LOADING;
      end if;
      if sta013request = '1' and (state = LOADING or state = WAITING) then
        state <= CLOCKING;
      end if;
      if sta013request = '0' and state = CLOCKING then
        state <= WAITING;
      end if;
      if bitcountzero = '1' and state = CLOCKING then
        state <= IDLE;
      end if;
    end if;
  end process statemachine;
  
  -- load, re-arranging data to big endian from ide little endian
  -- shift out msb first
  sta013data <= dataout(16);
  datain <= "0" & d16_in(7 downto 0) & d16_in(15 downto 8) when state = LOADING else dataout(15 downto 0) & "0";
  data : process(clk, reset, datain, state)
  begin
    if reset = '1' then
      dataout <= (others => '0');
    elsif clk'event and clk = '1' then
      if (state = CLOCKING and bitcount(0) = '1') or state = LOADING then
        dataout <= datain;
      end if;
    end if;	
  end process data;

end behavior; 
